from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class V1cnlterminationQueryQueriedMerchantPrincipalsAddress(BaseModel):
    """Details of the owner's business address.

    :param address_line_1: Line 1 of the address, such as a building number and street name.\<BR\>String with letters, numbers, numeric letters, and spaces. Applicable street details in this order: building number/name, street number and name, shop number, floor number.\<BR\>Length: 1-60, defaults to None
    :type address_line_1: str, optional
    :param address_line_2: Line 2 of the address, such as a building number and street name.\<BR\>String with letters, numbers, numeric letters, and spaces.\<BR\>Length: 1-60, defaults to None
    :type address_line_2: str, optional
    :param city: City portion of the address.\<BR\>String with letters, numbers, and spaces.\<BR\>Length: 1-20, defaults to None
    :type city: str, optional
    :param country: Code for the country.\<BR\>The two-letter ISO 3166-1 ALPHA-2 country code.\<BR\>Length: 2, defaults to None
    :type country: str, optional
    :param postal_code: Postal code portion of the address.\<BR\>String with letters, numbers, and spaces.\<BR\>Length: 1-10, defaults to None
    :type postal_code: str, optional
    """

    def __init__(
        self,
        address_line_1: str = None,
        address_line_2: str = None,
        city: str = None,
        country: str = None,
        postal_code: str = None,
    ):
        """Details of the owner's business address.

        :param address_line_1: Line 1 of the address, such as a building number and street name.\<BR\>String with letters, numbers, numeric letters, and spaces. Applicable street details in this order: building number/name, street number and name, shop number, floor number.\<BR\>Length: 1-60, defaults to None
        :type address_line_1: str, optional
        :param address_line_2: Line 2 of the address, such as a building number and street name.\<BR\>String with letters, numbers, numeric letters, and spaces.\<BR\>Length: 1-60, defaults to None
        :type address_line_2: str, optional
        :param city: City portion of the address.\<BR\>String with letters, numbers, and spaces.\<BR\>Length: 1-20, defaults to None
        :type city: str, optional
        :param country: Code for the country.\<BR\>The two-letter ISO 3166-1 ALPHA-2 country code.\<BR\>Length: 2, defaults to None
        :type country: str, optional
        :param postal_code: Postal code portion of the address.\<BR\>String with letters, numbers, and spaces.\<BR\>Length: 1-10, defaults to None
        :type postal_code: str, optional
        """
        self.address_line_1 = self._define_str(
            "address_line_1", address_line_1, nullable=True
        )
        self.address_line_2 = self._define_str(
            "address_line_2", address_line_2, nullable=True
        )
        self.city = self._define_str("city", city, nullable=True)
        self.country = self._define_str("country", country, nullable=True)
        self.postal_code = self._define_str("postal_code", postal_code, nullable=True)



from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1ordersorder_idreturns_items import V1ordersorderIdreturnsItems


@JsonMap({})
class OrderIdReturnsBody(BaseModel):
    """OrderIdReturnsBody

    :param order_id: order_id, defaults to None
    :type order_id: str, optional
    :param items: items
    :type items: List[V1ordersorderIdreturnsItems]
    """

    def __init__(self, items: List[V1ordersorderIdreturnsItems], order_id: str = None):
        """OrderIdReturnsBody

        :param order_id: order_id, defaults to None
        :type order_id: str, optional
        :param items: items
        :type items: List[V1ordersorderIdreturnsItems]
        """
        self.order_id = self._define_str("order_id", order_id, nullable=True)
        self.items = self._define_list(items, V1ordersorderIdreturnsItems)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PaymentsGroupPaymentsBody(BaseModel):
    """PaymentsGroupPaymentsBody

    :param description: Description of the group payment, defaults to None
    :type description: str, optional
    :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param payments: Array of 'payment' objects. For details about the fields in the 'payment' object, see [Create Payment](https://docs.rapyd.net/en/create-payment.html).
    :type payments: dict
    """

    def __init__(
        self,
        payments: dict,
        description: str = None,
        merchant_reference_id: str = None,
        metadata: dict = None,
    ):
        """PaymentsGroupPaymentsBody

        :param description: Description of the group payment, defaults to None
        :type description: str, optional
        :param merchant_reference_id: ID defined by the client. Limited to 255 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param payments: Array of 'payment' objects. For details about the fields in the 'payment' object, see [Create Payment](https://docs.rapyd.net/en/create-payment.html).
        :type payments: dict
        """
        self.description = self._define_str("description", description, nullable=True)
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.metadata = metadata
        self.payments = payments



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class CardNetwork(Enum):
    """An enumeration representing different categories.

    :cvar MASTERCARD: "MASTERCARD"
    :vartype MASTERCARD: str
    :cvar VISA: "VISA"
    :vartype VISA: str
    """

    MASTERCARD = "MASTERCARD"
    VISA = "VISA"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, CardNetwork._member_map_.values()))


@JsonMap({})
class InlineResponse200_119DataMatches(BaseModel):
    """Describes the results of the query.

    :param card_network: Name of the card network termination database with a match., defaults to None
    :type card_network: CardNetwork, optional
    :param exact_match: List of the data points that match this query exactly. Array of strings., defaults to None
    :type exact_match: dict, optional
    :param match_type: Type of match. One of the following:\<BR\>* **query match** - This merchant query matches elements of a previous merchant query in the card network termination database.\<BR\>This means that the queried merchant shares data with a merchant that was previously searched for.\<BR\>* **registered match** - This merchant query matches elements of a merchant termination registration in the card network termination database. This means that an acquirer terminated the contract of a merchant that shares data with the queried merchant., defaults to None
    :type match_type: str, optional
    :param partial_match: List of the data points that partly match the query. Array of strings., defaults to None
    :type partial_match: dict, optional
    """

    def __init__(
        self,
        card_network: CardNetwork = None,
        exact_match: dict = None,
        match_type: str = None,
        partial_match: dict = None,
    ):
        """Describes the results of the query.

        :param card_network: Name of the card network termination database with a match., defaults to None
        :type card_network: CardNetwork, optional
        :param exact_match: List of the data points that match this query exactly. Array of strings., defaults to None
        :type exact_match: dict, optional
        :param match_type: Type of match. One of the following:\<BR\>* **query match** - This merchant query matches elements of a previous merchant query in the card network termination database.\<BR\>This means that the queried merchant shares data with a merchant that was previously searched for.\<BR\>* **registered match** - This merchant query matches elements of a merchant termination registration in the card network termination database. This means that an acquirer terminated the contract of a merchant that shares data with the queried merchant., defaults to None
        :type match_type: str, optional
        :param partial_match: List of the data points that partly match the query. Array of strings., defaults to None
        :type partial_match: dict, optional
        """
        self.card_network = (
            self._enum_matching(card_network, CardNetwork.list(), "card_network")
            if card_network
            else None
        )
        self.exact_match = exact_match
        self.match_type = self._define_str("match_type", match_type, nullable=True)
        self.partial_match = partial_match



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .status_1 import Status1
from .dispute import Dispute


@JsonMap({})
class InlineResponse200_35(BaseModel):
    """InlineResponse200_35

    :param status: status, defaults to None
    :type status: Status1, optional
    :param data: Describes the fields contained in REST messages and webhooks for disputes of payments., defaults to None
    :type data: Dispute, optional
    """

    def __init__(self, status: Status1 = None, data: Dispute = None):
        """InlineResponse200_35

        :param status: status, defaults to None
        :type status: Status1, optional
        :param data: Describes the fields contained in REST messages and webhooks for disputes of payments., defaults to None
        :type data: Dispute, optional
        """
        self.status = self._define_object(status, Status1)
        self.data = self._define_object(data, Dispute)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PayoutsPayoutIdBody(BaseModel):
    """PayoutsPayoutIdBody

    :param description: Description of the payout., defaults to None
    :type description: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    """

    def __init__(self, description: str = None, metadata: dict = None):
        """PayoutsPayoutIdBody

        :param description: Description of the payout., defaults to None
        :type description: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        """
        self.description = self._define_str("description", description, nullable=True)
        self.metadata = metadata



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class FinancialImpact(Enum):
    """An enumeration representing different categories.

    :cvar CREDIT: "credit"
    :vartype CREDIT: str
    :cvar DEBIT: "debit"
    :vartype DEBIT: str
    """

    CREDIT = "credit"
    DEBIT = "debit"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, FinancialImpact._member_map_.values()))


@JsonMap({})
class CardsAuthorizationBody(BaseModel):
    """CardsAuthorizationBody

    :param amount: The amount of the authorization, in units of the `currency` defined in currency. Decimal.
    :type amount: float
    :param card_id: ID of the card. String starting with **card_**.
    :type card_id: str
    :param currency: The currency supported by the card. Three-letter ISO 4217 code.
    :type currency: str
    :param financial_impact: Indicates the financial impact of the transaction.
    :type financial_impact: FinancialImpact
    """

    def __init__(
        self,
        amount: float,
        card_id: str,
        currency: str,
        financial_impact: FinancialImpact,
    ):
        """CardsAuthorizationBody

        :param amount: The amount of the authorization, in units of the `currency` defined in currency. Decimal.
        :type amount: float
        :param card_id: ID of the card. String starting with **card_**.
        :type card_id: str
        :param currency: The currency supported by the card. Three-letter ISO 4217 code.
        :type currency: str
        :param financial_impact: Indicates the financial impact of the transaction.
        :type financial_impact: FinancialImpact
        """
        self.amount = amount
        self.card_id = card_id
        self.currency = currency
        self.financial_impact = self._enum_matching(
            financial_impact, FinancialImpact.list(), "financial_impact"
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .beneficiary import Beneficiary


@JsonMap({})
class InlineResponse200_27Data(BaseModel):
    """InlineResponse200_27Data

    :param beneficiary: beneficiary, defaults to None
    :type beneficiary: Beneficiary, optional
    :param validated: validation status, defaults to None
    :type validated: bool, optional
    """

    def __init__(self, beneficiary: Beneficiary = None, validated: bool = None):
        """InlineResponse200_27Data

        :param beneficiary: beneficiary, defaults to None
        :type beneficiary: Beneficiary, optional
        :param validated: validation status, defaults to None
        :type validated: bool, optional
        """
        self.beneficiary = self._define_object(beneficiary, Beneficiary)
        self.validated = validated



from enum import Enum


class PayoutStatus(Enum):
    """An enumeration representing different categories.

    :cvar CREATED: "Created"
    :vartype CREATED: str
    :cvar COMPLETED: "Completed"
    :vartype COMPLETED: str
    :cvar CANCELED: "Canceled"
    :vartype CANCELED: str
    :cvar REJECTED: "Rejected"
    :vartype REJECTED: str
    :cvar ERROR: "Error"
    :vartype ERROR: str
    :cvar CONFIRMATION: "Confirmation"
    :vartype CONFIRMATION: str
    :cvar EXPIRED: "Expired"
    :vartype EXPIRED: str
    :cvar RETURNED: "Returned"
    :vartype RETURNED: str
    """

    CREATED = "Created"
    COMPLETED = "Completed"
    CANCELED = "Canceled"
    REJECTED = "Rejected"
    ERROR = "Error"
    CONFIRMATION = "Confirmation"
    EXPIRED = "Expired"
    RETURNED = "Returned"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, PayoutStatus._member_map_.values()))



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .plan import Plan


@JsonMap({"id_": "id"})
class SubscriptionItem(BaseModel):
    """SubscriptionItem

    :param created: The time the subscription item was created, in Unix time. Response only., defaults to None
    :type created: float, optional
    :param id_: ID of the Subscription Item object. String starting with subi_., defaults to None
    :type id_: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param plan: Describes the pricing structure for the invoice item. For details of the fields in the `plan` object, see 'Create Plan'., defaults to None
    :type plan: Plan, optional
    :param quantity: The number of units of the service defined in the plan. Integer. This number can be updated during the billing cycle using Update Subscription or Update Subscription Item., defaults to None
    :type quantity: float, optional
    :param subscription_id: ID of the subscription that this item belongs to. String starting with sub_., defaults to None
    :type subscription_id: str, optional
    """

    def __init__(
        self,
        created: float = None,
        id_: str = None,
        metadata: dict = None,
        plan: Plan = None,
        quantity: float = None,
        subscription_id: str = None,
    ):
        """SubscriptionItem

        :param created: The time the subscription item was created, in Unix time. Response only., defaults to None
        :type created: float, optional
        :param id_: ID of the Subscription Item object. String starting with subi_., defaults to None
        :type id_: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param plan: Describes the pricing structure for the invoice item. For details of the fields in the `plan` object, see 'Create Plan'., defaults to None
        :type plan: Plan, optional
        :param quantity: The number of units of the service defined in the plan. Integer. This number can be updated during the billing cycle using Update Subscription or Update Subscription Item., defaults to None
        :type quantity: float, optional
        :param subscription_id: ID of the subscription that this item belongs to. String starting with sub_., defaults to None
        :type subscription_id: str, optional
        """
        self.created = self._define_number("created", created, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.metadata = metadata
        self.plan = self._define_object(plan, Plan)
        self.quantity = self._define_number("quantity", quantity, nullable=True)
        self.subscription_id = self._define_str(
            "subscription_id", subscription_id, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PaymentsCompletePaymentBody(BaseModel):
    """PaymentsCompletePaymentBody

    :param token: ID of the payment to complete. String starting with **payment_**., defaults to None
    :type token: str, optional
    :param param1: Depends on the type of payment method. \<BR\> * bank_redirect - **rapyd** \<BR\>* bank_transfer - The value returned in the code field of the `textual_codes` object. You can find this field in the response to the 'Create Payment' request. If the `code` field is empty, `param1` is not required.\<BR\>* cash - Not required.\<BR\>* ewallet - Not required., defaults to None
    :type param1: str, optional
    :param param2: Depends on the type of payment method. \<BR\> * bank_redirect - **success** \<BR\>* bank_transfer -  Original payment amount. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015.\<BR\>* cash - Not required.\<BR\>* ewallet - Not required., defaults to None
    :type param2: str, optional
    """

    def __init__(self, token: str = None, param1: str = None, param2: str = None):
        """PaymentsCompletePaymentBody

        :param token: ID of the payment to complete. String starting with **payment_**., defaults to None
        :type token: str, optional
        :param param1: Depends on the type of payment method. \<BR\> * bank_redirect - **rapyd** \<BR\>* bank_transfer - The value returned in the code field of the `textual_codes` object. You can find this field in the response to the 'Create Payment' request. If the `code` field is empty, `param1` is not required.\<BR\>* cash - Not required.\<BR\>* ewallet - Not required., defaults to None
        :type param1: str, optional
        :param param2: Depends on the type of payment method. \<BR\> * bank_redirect - **success** \<BR\>* bank_transfer -  Original payment amount. Decimal, including the correct number of decimal places for the currency exponent, as defined in ISO 2417:2015.\<BR\>* cash - Not required.\<BR\>* ewallet - Not required., defaults to None
        :type param2: str, optional
        """
        self.token = self._define_str("token", token, nullable=True)
        self.param1 = self._define_str("param1", param1, nullable=True)
        self.param2 = self._define_str("param2", param2, nullable=True)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .status_1 import Status1
from .merchant_customer_support import MerchantCustomerSupport
from .hosted_page_status import HostedPageStatus
from .payment_params import PaymentParams
from .customer_payment_method import CustomerPaymentMethod, CustomerPaymentMethodGuard


@JsonMap({})
class Card(BaseModel):
    """Card

    :param name: The name of the person that the card is issued to., defaults to None
    :type name: str, optional
    """

    def __init__(self, name: str = None):
        """Card

        :param name: The name of the person that the card is issued to., defaults to None
        :type name: str, optional
        """
        self.name = self._define_str("name", name, nullable=True)


@JsonMap({"id_": "id"})
class InlineResponse200_74Data(BaseModel):
    """InlineResponse200_74Data

    :param language: Determines the default language of the hosted page. When this parameter is null, the language of the user's browser is used. If the language of the user's browser cannot be determined, the default language is English, defaults to None
    :type language: str, optional
    :param merchant_alias: Reserved, the default name is Rapyd, defaults to None
    :type merchant_alias: str, optional
    :param merchant_color: Color of the call-to-action (CTA) button on the hosted page. To configure this field, use the Client Portal, defaults to None
    :type merchant_color: str, optional
    :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
    :type merchant_customer_support: MerchantCustomerSupport, optional
    :param merchant_logo: URL for the image of the client's logo. To configure this field, use the Client Portal, defaults to None
    :type merchant_logo: str, optional
    :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when cancel_url or complete_url or both fields is unset. To configure this field, use the Client Portal, defaults to None
    :type merchant_website: str, optional
    :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
    :type redirect_url: str, optional
    :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Portal, defaults to None
    :type merchant_terms: str, optional
    :param cancel_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
    :type cancel_url: str, optional
    :param complete_url: URL where the customer is redirected after pressing Close to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
    :type complete_url: str, optional
    :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers. To configure this field, use the Client Portal., defaults to None
    :type merchant_privacy_policy: str, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If page_expiration is not set, the hosted page expires 14 days after creation. Range is 1 minute to 30 days., defaults to None
    :type page_expiration: float, optional
    :param status: Status of the hosted page. One of the following: NEW - The hosted page was created. DON - Done. The card was added to the customer profile. EXP - The hosted page expired. , defaults to None
    :type status: HostedPageStatus, optional
    :param card: card, defaults to None
    :type card: Card, optional
    :param country: country, defaults to None
    :type country: str, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency: str, optional
    :param customer: ID of the customer, a string starting with **cus_**., defaults to None
    :type customer: str, optional
    :param id_: ID of the card token hosted page, a string starting with hp_card_, defaults to None
    :type id_: str, optional
    :param payment_method_type: Limits the page to a specific type of payment method. For example, dk_visa_card, defaults to None
    :type payment_method_type: str, optional
    :param payment_params: payment_params, defaults to None
    :type payment_params: PaymentParams, optional
    :param customer_card_payment_methods: customer_card_payment_methods, defaults to None
    :type customer_card_payment_methods: CustomerPaymentMethod, optional
    """

    def __init__(
        self,
        language: str = None,
        merchant_alias: str = None,
        merchant_color: str = None,
        merchant_customer_support: MerchantCustomerSupport = None,
        merchant_logo: str = None,
        merchant_website: str = None,
        redirect_url: str = None,
        merchant_terms: str = None,
        cancel_url: str = None,
        complete_url: str = None,
        merchant_privacy_policy: str = None,
        page_expiration: float = None,
        status: HostedPageStatus = None,
        card: Card = None,
        country: str = None,
        currency: str = None,
        customer: str = None,
        id_: str = None,
        payment_method_type: str = None,
        payment_params: PaymentParams = None,
        customer_card_payment_methods: CustomerPaymentMethod = None,
    ):
        """InlineResponse200_74Data

        :param language: Determines the default language of the hosted page. When this parameter is null, the language of the user's browser is used. If the language of the user's browser cannot be determined, the default language is English, defaults to None
        :type language: str, optional
        :param merchant_alias: Reserved, the default name is Rapyd, defaults to None
        :type merchant_alias: str, optional
        :param merchant_color: Color of the call-to-action (CTA) button on the hosted page. To configure this field, use the Client Portal, defaults to None
        :type merchant_color: str, optional
        :param merchant_customer_support: Contains details of the client’s customer support. To configure these fields, use the Client Portal., defaults to None
        :type merchant_customer_support: MerchantCustomerSupport, optional
        :param merchant_logo: URL for the image of the client's logo. To configure this field, use the Client Portal, defaults to None
        :type merchant_logo: str, optional
        :param merchant_website: The URL where the customer is redirected after exiting the hosted page. Relevant when cancel_url or complete_url or both fields is unset. To configure this field, use the Client Portal, defaults to None
        :type merchant_website: str, optional
        :param redirect_url: URL of the hosted page that is shown to the customer., defaults to None
        :type redirect_url: str, optional
        :param merchant_terms: URL for the client's terms and conditions. To configure this field, use the Client Portal, defaults to None
        :type merchant_terms: str, optional
        :param cancel_url: URL where the customer is redirected after pressing Back to Website to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
        :type cancel_url: str, optional
        :param complete_url: URL where the customer is redirected after pressing Close to exit the hosted page. This URL overrides the merchant_website URL. Does not support localhost URLs., defaults to None
        :type complete_url: str, optional
        :param merchant_privacy_policy: URL for the terms and conditions of the agreement between the client and the client’s customers. To configure this field, use the Client Portal., defaults to None
        :type merchant_privacy_policy: str, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If page_expiration is not set, the hosted page expires 14 days after creation. Range is 1 minute to 30 days., defaults to None
        :type page_expiration: float, optional
        :param status: Status of the hosted page. One of the following: NEW - The hosted page was created. DON - Done. The card was added to the customer profile. EXP - The hosted page expired. , defaults to None
        :type status: HostedPageStatus, optional
        :param card: card, defaults to None
        :type card: Card, optional
        :param country: country, defaults to None
        :type country: str, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency: str, optional
        :param customer: ID of the customer, a string starting with **cus_**., defaults to None
        :type customer: str, optional
        :param id_: ID of the card token hosted page, a string starting with hp_card_, defaults to None
        :type id_: str, optional
        :param payment_method_type: Limits the page to a specific type of payment method. For example, dk_visa_card, defaults to None
        :type payment_method_type: str, optional
        :param payment_params: payment_params, defaults to None
        :type payment_params: PaymentParams, optional
        :param customer_card_payment_methods: customer_card_payment_methods, defaults to None
        :type customer_card_payment_methods: CustomerPaymentMethod, optional
        """
        self.language = self._define_str("language", language, nullable=True)
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_color = self._define_str(
            "merchant_color", merchant_color, nullable=True
        )
        self.merchant_customer_support = self._define_object(
            merchant_customer_support, MerchantCustomerSupport
        )
        self.merchant_logo = self._define_str(
            "merchant_logo", merchant_logo, nullable=True
        )
        self.merchant_website = self._define_str(
            "merchant_website", merchant_website, nullable=True
        )
        self.redirect_url = self._define_str(
            "redirect_url", redirect_url, nullable=True
        )
        self.merchant_terms = self._define_str(
            "merchant_terms", merchant_terms, nullable=True
        )
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.merchant_privacy_policy = self._define_str(
            "merchant_privacy_policy", merchant_privacy_policy, nullable=True
        )
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.status = (
            self._enum_matching(status, HostedPageStatus.list(), "status")
            if status
            else None
        )
        self.card = self._define_object(card, Card)
        self.country = self._define_str(
            "country",
            country,
            nullable=True,
            pattern="Name of the country. Two-letter ISO 3166-1 alpha-2 code.",
        )
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.customer = self._define_str("customer", customer, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.payment_method_type = self._define_str(
            "payment_method_type", payment_method_type, nullable=True
        )
        self.payment_params = self._define_object(payment_params, PaymentParams)
        self.customer_card_payment_methods = CustomerPaymentMethodGuard.return_one_of(
            customer_card_payment_methods
        )


@JsonMap({})
class InlineResponse200_74(BaseModel):
    """InlineResponse200_74

    :param data: data, defaults to None
    :type data: InlineResponse200_74Data, optional
    :param status: status, defaults to None
    :type status: Status1, optional
    """

    def __init__(self, data: InlineResponse200_74Data = None, status: Status1 = None):
        """InlineResponse200_74

        :param data: data, defaults to None
        :type data: InlineResponse200_74Data, optional
        :param status: status, defaults to None
        :type status: Status1, optional
        """
        self.data = self._define_object(data, InlineResponse200_74Data)
        self.status = self._define_object(status, Status1)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class InlineResponse200_68Data(BaseModel):
    """InlineResponse200_68Data

    :param account_id: ID of the Rapyd wallet account. UUID., defaults to None
    :type account_id: str, optional
    :param amount: Amount of the transfer, in units of the currency specified in `currency`., defaults to None
    :type amount: float, optional
    :param balance_type: Indicates the type of balance within the Rapyd wallet account., defaults to None
    :type balance_type: str, optional
    :param currency: The currency of the transfer. Three-letter ISO 4217 code., defaults to None
    :type currency: str, optional
    :param id_: ID of the funds transfer transaction. UUID., defaults to None
    :type id_: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: str, optional
    :param phone_number: Phone number of the Rapyd wallet., defaults to None
    :type phone_number: str, optional
    """

    def __init__(
        self,
        account_id: str = None,
        amount: float = None,
        balance_type: str = None,
        currency: str = None,
        id_: str = None,
        metadata: str = None,
        phone_number: str = None,
    ):
        """InlineResponse200_68Data

        :param account_id: ID of the Rapyd wallet account. UUID., defaults to None
        :type account_id: str, optional
        :param amount: Amount of the transfer, in units of the currency specified in `currency`., defaults to None
        :type amount: float, optional
        :param balance_type: Indicates the type of balance within the Rapyd wallet account., defaults to None
        :type balance_type: str, optional
        :param currency: The currency of the transfer. Three-letter ISO 4217 code., defaults to None
        :type currency: str, optional
        :param id_: ID of the funds transfer transaction. UUID., defaults to None
        :type id_: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: str, optional
        :param phone_number: Phone number of the Rapyd wallet., defaults to None
        :type phone_number: str, optional
        """
        self.account_id = self._define_str("account_id", account_id, nullable=True)
        self.amount = self._define_number("amount", amount, nullable=True)
        self.balance_type = self._define_str(
            "balance_type", balance_type, nullable=True
        )
        self.currency = self._define_str("currency", currency, nullable=True)
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.metadata = self._define_str("metadata", metadata, nullable=True)
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1hosteddisbursebeneficiary_beneficiary_optional_fields import (
    V1hosteddisbursebeneficiaryBeneficiaryOptionalFields,
)


class DisburseBeneficiaryBodyBeneficiaryEntityType(Enum):
    """An enumeration representing different categories.

    :cvar COMPANY: "company"
    :vartype COMPANY: str
    :cvar INDIVIDUAL: "individual"
    :vartype INDIVIDUAL: str
    """

    COMPANY = "company"
    INDIVIDUAL = "individual"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                DisburseBeneficiaryBodyBeneficiaryEntityType._member_map_.values(),
            )
        )


class DisburseBeneficiaryBodyBeneficiaryExtendedFields(Enum):
    """An enumeration representing different categories.

    :cvar TRUE: "true"
    :vartype TRUE: str
    :cvar FALSE: "false"
    :vartype FALSE: str
    """

    TRUE = "true"
    FALSE = "false"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                DisburseBeneficiaryBodyBeneficiaryExtendedFields._member_map_.values(),
            )
        )


class DisburseBeneficiaryBodyCategory(Enum):
    """An enumeration representing different categories.

    :cvar BANK: "bank"
    :vartype BANK: str
    :cvar CARD: "card"
    :vartype CARD: str
    """

    BANK = "bank"
    CARD = "card"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, DisburseBeneficiaryBodyCategory._member_map_.values()
            )
        )


class DisburseBeneficiaryBodySenderEntityType(Enum):
    """An enumeration representing different categories.

    :cvar COMPANY: "company"
    :vartype COMPANY: str
    :cvar INDIVIDUAL: "individual"
    :vartype INDIVIDUAL: str
    """

    COMPANY = "company"
    INDIVIDUAL = "individual"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                DisburseBeneficiaryBodySenderEntityType._member_map_.values(),
            )
        )


@JsonMap({})
class DisburseBeneficiaryBody(BaseModel):
    """DisburseBeneficiaryBody

    :param beneficiary_country: The two-letter ISO 3166-1 ALPHA-2 code for the country of the beneficiary’s bank account. The two-letter prefix of the payout method type matches the beneficiary country code., defaults to None
    :type beneficiary_country: str, optional
    :param beneficiary_entity_type: Type of entity for the beneficiary. One of the following:
    :type beneficiary_entity_type: DisburseBeneficiaryBodyBeneficiaryEntityType
    :param beneficiary_extended_fields: When the value is **true**, the hosted tokenization page displays additional required fields.\<BR\> * Additional required fields when `beneficiary_entity_type` is **individual**: `address`, `city`, `country`, `date_of_birth`, `first_name`, `gender`, `identification_type`, `identification_value`, `last_name`, `nationality`. \<BR\> * Additional required fields when `beneficiary_entity_type` is **company**: `address`, `city`, `company_name`, `country`, `country_of_incorporation`, `date_of_incorporation`, `identification_type`, `identification_value`.\<BR\> For more information on the additional required fields, see 'Create Extended Beneficiary'.\<BR\> **Note**: Currently only available in the sandbox., defaults to None
    :type beneficiary_extended_fields: DisburseBeneficiaryBodyBeneficiaryExtendedFields, optional
    :param beneficiary_optional_fields: Additional information about the beneficiary., defaults to None
    :type beneficiary_optional_fields: List[V1hosteddisbursebeneficiaryBeneficiaryOptionalFields], optional
    :param category: The category of payout method. One of the following: \<BR\> * **bank**\<BR\> * **card** - US only., defaults to None
    :type category: DisburseBeneficiaryBodyCategory, optional
    :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
    :type complete_url: str, optional
    :param language: Determines the default language of the hosted page. For a list of values, see 'List Supported Languages'.\<BR\> * When this parameter is null, the language of the user's browser is used.\<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
    :type language: str, optional
    :param merchant_alias: Client's name., defaults to None
    :type merchant_alias: str, optional
    :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation. \<BR\> **Range**: 1 minute to 30 days., defaults to None
    :type page_expiration: float, optional
    :param payout_currency: Currency accepted by the beneficiary’s bank account. Three-letter ISO 4217 code., defaults to None
    :type payout_currency: str, optional
    :param payout_method_types_exclude: List of payout methods that are excluded from display on the beneficiary tokenization page., defaults to None
    :type payout_method_types_exclude: List[dict], optional
    :param payout_method_types_include: Array of strings. List of payout methods that are displayed on the beneficiary tokenization page., defaults to None
    :type payout_method_types_include: List[dict], optional
    :param sender_country: The two-letter ISO 3166-1 ALPHA-2 code for the sender’s country.
    :type sender_country: str
    :param sender_currency: Currency paid from the sender’s wallet. Three-letter ISO 4217 code., defaults to None
    :type sender_currency: str, optional
    :param sender_entity_type: Type of entity for the sender.
    :type sender_entity_type: DisburseBeneficiaryBodySenderEntityType
    """

    def __init__(
        self,
        beneficiary_entity_type: DisburseBeneficiaryBodyBeneficiaryEntityType,
        sender_country: str,
        sender_entity_type: DisburseBeneficiaryBodySenderEntityType,
        beneficiary_country: str = None,
        beneficiary_extended_fields: DisburseBeneficiaryBodyBeneficiaryExtendedFields = None,
        beneficiary_optional_fields: List[
            V1hosteddisbursebeneficiaryBeneficiaryOptionalFields
        ] = None,
        category: DisburseBeneficiaryBodyCategory = None,
        complete_url: str = None,
        language: str = None,
        merchant_alias: str = None,
        merchant_reference_id: str = None,
        page_expiration: float = None,
        payout_currency: str = None,
        payout_method_types_exclude: List[dict] = None,
        payout_method_types_include: List[dict] = None,
        sender_currency: str = None,
    ):
        """DisburseBeneficiaryBody

        :param beneficiary_country: The two-letter ISO 3166-1 ALPHA-2 code for the country of the beneficiary’s bank account. The two-letter prefix of the payout method type matches the beneficiary country code., defaults to None
        :type beneficiary_country: str, optional
        :param beneficiary_entity_type: Type of entity for the beneficiary. One of the following:
        :type beneficiary_entity_type: DisburseBeneficiaryBodyBeneficiaryEntityType
        :param beneficiary_extended_fields: When the value is **true**, the hosted tokenization page displays additional required fields.\<BR\> * Additional required fields when `beneficiary_entity_type` is **individual**: `address`, `city`, `country`, `date_of_birth`, `first_name`, `gender`, `identification_type`, `identification_value`, `last_name`, `nationality`. \<BR\> * Additional required fields when `beneficiary_entity_type` is **company**: `address`, `city`, `company_name`, `country`, `country_of_incorporation`, `date_of_incorporation`, `identification_type`, `identification_value`.\<BR\> For more information on the additional required fields, see 'Create Extended Beneficiary'.\<BR\> **Note**: Currently only available in the sandbox., defaults to None
        :type beneficiary_extended_fields: DisburseBeneficiaryBodyBeneficiaryExtendedFields, optional
        :param beneficiary_optional_fields: Additional information about the beneficiary., defaults to None
        :type beneficiary_optional_fields: List[V1hosteddisbursebeneficiaryBeneficiaryOptionalFields], optional
        :param category: The category of payout method. One of the following: \<BR\> * **bank**\<BR\> * **card** - US only., defaults to None
        :type category: DisburseBeneficiaryBodyCategory, optional
        :param complete_url: URL where the customer is redirected after pressing **Close** to exit the hosted page. This URL overrides the `merchant_website` URL. Does not support localhost URLs., defaults to None
        :type complete_url: str, optional
        :param language: Determines the default language of the hosted page. For a list of values, see 'List Supported Languages'.\<BR\> * When this parameter is null, the language of the user's browser is used.\<BR\> * If the language of the user's browser cannot be determined, the default language is English., defaults to None
        :type language: str, optional
        :param merchant_alias: Client's name., defaults to None
        :type merchant_alias: str, optional
        :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit: 45 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param page_expiration: End of the time when the customer can use the hosted page, in Unix time. If `page_expiration` is not set, the hosted page expires 14 days after creation. \<BR\> **Range**: 1 minute to 30 days., defaults to None
        :type page_expiration: float, optional
        :param payout_currency: Currency accepted by the beneficiary’s bank account. Three-letter ISO 4217 code., defaults to None
        :type payout_currency: str, optional
        :param payout_method_types_exclude: List of payout methods that are excluded from display on the beneficiary tokenization page., defaults to None
        :type payout_method_types_exclude: List[dict], optional
        :param payout_method_types_include: Array of strings. List of payout methods that are displayed on the beneficiary tokenization page., defaults to None
        :type payout_method_types_include: List[dict], optional
        :param sender_country: The two-letter ISO 3166-1 ALPHA-2 code for the sender’s country.
        :type sender_country: str
        :param sender_currency: Currency paid from the sender’s wallet. Three-letter ISO 4217 code., defaults to None
        :type sender_currency: str, optional
        :param sender_entity_type: Type of entity for the sender.
        :type sender_entity_type: DisburseBeneficiaryBodySenderEntityType
        """
        self.beneficiary_country = self._define_str(
            "beneficiary_country", beneficiary_country, nullable=True
        )
        self.beneficiary_entity_type = self._enum_matching(
            beneficiary_entity_type,
            DisburseBeneficiaryBodyBeneficiaryEntityType.list(),
            "beneficiary_entity_type",
        )
        self.beneficiary_extended_fields = (
            self._enum_matching(
                beneficiary_extended_fields,
                DisburseBeneficiaryBodyBeneficiaryExtendedFields.list(),
                "beneficiary_extended_fields",
            )
            if beneficiary_extended_fields
            else None
        )
        self.beneficiary_optional_fields = self._define_list(
            beneficiary_optional_fields,
            V1hosteddisbursebeneficiaryBeneficiaryOptionalFields,
        )
        self.category = (
            self._enum_matching(
                category, DisburseBeneficiaryBodyCategory.list(), "category"
            )
            if category
            else None
        )
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.language = self._define_str("language", language, nullable=True)
        self.merchant_alias = self._define_str(
            "merchant_alias", merchant_alias, nullable=True
        )
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.page_expiration = self._define_number(
            "page_expiration", page_expiration, nullable=True
        )
        self.payout_currency = self._define_str(
            "payout_currency", payout_currency, nullable=True
        )
        self.payout_method_types_exclude = payout_method_types_exclude
        self.payout_method_types_include = payout_method_types_include
        self.sender_country = sender_country
        self.sender_currency = self._define_str(
            "sender_currency", sender_currency, nullable=True
        )
        self.sender_entity_type = self._enum_matching(
            sender_entity_type,
            DisburseBeneficiaryBodySenderEntityType.list(),
            "sender_entity_type",
        )



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .v1_addresses_body import V1AddressesBody


@JsonMap({})
class EwalletIdContactsBody(BaseModel):
    """EwalletIdContactsBody

    :param address: address associated with this specific Rapyd entity Payment/Customer etc..., defaults to None
    :type address: V1AddressesBody, optional
    :param contact_type: Type of contact: personal.
    :type contact_type: str
    :param country: 2-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase., defaults to None
    :type country: str, optional
    :param date_of_birth: Date of birth of the individual. Format: MM/DD/YYYY, defaults to None
    :type date_of_birth: str, optional
    :param email: Email address for the contact., defaults to None
    :type email: str, optional
    :param first_name: First name of the contact., defaults to None
    :type first_name: str, optional
    :param last_name: Family name of the personal contact or primary person associated with the business contact. For a personal contact type, alphabetic characters and spaces. Required for issuing a card to a personal contact. , defaults to None
    :type last_name: str, optional
    :param gender: Gender of the contact. One of the following values: male, female, other, not_applicable, defaults to None
    :type gender: str, optional
    :param house_type: Description of the type of residency. One of the following values: lease, live_with_family, own, owner, month_to_month, housing_project, defaults to None
    :type house_type: str, optional
    :param identification_number: ID number as shown by the ID document. Alphanumeric string, defaults to None
    :type identification_number: str, optional
    :param identification_type: Type of the identification document, defaults to None
    :type identification_type: str, optional
    :param marital_status: Marital status of the contact. One of the following values: married, single, divorced, widowed, cohabiting, not_applicable, defaults to None
    :type marital_status: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: str, optional
    :param middle_name: Middle name of the personal contact or primary person associated with the business contact., defaults to None
    :type middle_name: str, optional
    :param mothers_name: Name of the contact's mother., defaults to None
    :type mothers_name: str, optional
    :param nationality: The citizenship of the contact. Two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase., defaults to None
    :type nationality: str, optional
    :param phone_number: Phone number of the contact in E.164 format., defaults to None
    :type phone_number: str, optional
    :param second_last_name: Second last name of the contact., defaults to None
    :type second_last_name: str, optional
    :param send_notifications: Determines whether Rapyd sends notifications to the contact. Default is false., defaults to None
    :type send_notifications: bool, optional
    :param contact_reference_id: Reference ID for the contact. Limited to 45 characters. Should be unique per merchant per e-wallet., defaults to None
    :type contact_reference_id: str, optional
    """

    def __init__(
        self,
        contact_type: str,
        address: V1AddressesBody = None,
        country: str = None,
        date_of_birth: str = None,
        email: str = None,
        first_name: str = None,
        last_name: str = None,
        gender: str = None,
        house_type: str = None,
        identification_number: str = None,
        identification_type: str = None,
        marital_status: str = None,
        metadata: str = None,
        middle_name: str = None,
        mothers_name: str = None,
        nationality: str = None,
        phone_number: str = None,
        second_last_name: str = None,
        send_notifications: bool = None,
        contact_reference_id: str = None,
    ):
        """EwalletIdContactsBody

        :param address: address associated with this specific Rapyd entity Payment/Customer etc..., defaults to None
        :type address: V1AddressesBody, optional
        :param contact_type: Type of contact: personal.
        :type contact_type: str
        :param country: 2-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase., defaults to None
        :type country: str, optional
        :param date_of_birth: Date of birth of the individual. Format: MM/DD/YYYY, defaults to None
        :type date_of_birth: str, optional
        :param email: Email address for the contact., defaults to None
        :type email: str, optional
        :param first_name: First name of the contact., defaults to None
        :type first_name: str, optional
        :param last_name: Family name of the personal contact or primary person associated with the business contact. For a personal contact type, alphabetic characters and spaces. Required for issuing a card to a personal contact. , defaults to None
        :type last_name: str, optional
        :param gender: Gender of the contact. One of the following values: male, female, other, not_applicable, defaults to None
        :type gender: str, optional
        :param house_type: Description of the type of residency. One of the following values: lease, live_with_family, own, owner, month_to_month, housing_project, defaults to None
        :type house_type: str, optional
        :param identification_number: ID number as shown by the ID document. Alphanumeric string, defaults to None
        :type identification_number: str, optional
        :param identification_type: Type of the identification document, defaults to None
        :type identification_type: str, optional
        :param marital_status: Marital status of the contact. One of the following values: married, single, divorced, widowed, cohabiting, not_applicable, defaults to None
        :type marital_status: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: str, optional
        :param middle_name: Middle name of the personal contact or primary person associated with the business contact., defaults to None
        :type middle_name: str, optional
        :param mothers_name: Name of the contact's mother., defaults to None
        :type mothers_name: str, optional
        :param nationality: The citizenship of the contact. Two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase., defaults to None
        :type nationality: str, optional
        :param phone_number: Phone number of the contact in E.164 format., defaults to None
        :type phone_number: str, optional
        :param second_last_name: Second last name of the contact., defaults to None
        :type second_last_name: str, optional
        :param send_notifications: Determines whether Rapyd sends notifications to the contact. Default is false., defaults to None
        :type send_notifications: bool, optional
        :param contact_reference_id: Reference ID for the contact. Limited to 45 characters. Should be unique per merchant per e-wallet., defaults to None
        :type contact_reference_id: str, optional
        """
        self.address = self._define_object(address, V1AddressesBody)
        self.contact_type = contact_type
        self.country = self._define_str("country", country, nullable=True)
        self.date_of_birth = self._define_str(
            "date_of_birth", date_of_birth, nullable=True
        )
        self.email = self._define_str("email", email, nullable=True)
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.gender = self._define_str("gender", gender, nullable=True)
        self.house_type = self._define_str("house_type", house_type, nullable=True)
        self.identification_number = self._define_str(
            "identification_number", identification_number, nullable=True
        )
        self.identification_type = self._define_str(
            "identification_type", identification_type, nullable=True
        )
        self.marital_status = self._define_str(
            "marital_status", marital_status, nullable=True
        )
        self.metadata = self._define_str("metadata", metadata, nullable=True)
        self.middle_name = self._define_str("middle_name", middle_name, nullable=True)
        self.mothers_name = self._define_str(
            "mothers_name", mothers_name, nullable=True
        )
        self.nationality = self._define_str("nationality", nationality, nullable=True)
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
        self.second_last_name = self._define_str(
            "second_last_name", second_last_name, nullable=True
        )
        self.send_notifications = send_notifications
        self.contact_reference_id = self._define_str(
            "contact_reference_id", contact_reference_id, nullable=True
        )



from __future__ import annotations
from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .virtual_account_transaction_response import VirtualAccountTransactionResponse


class VirtualAccountIssuingStatus(Enum):
    """An enumeration representing different categories.

    :cvar ACT: "ACT"
    :vartype ACT: str
    :cvar CLO: "CLO"
    :vartype CLO: str
    :cvar ERR: "ERR"
    :vartype ERR: str
    :cvar PEN: "PEN"
    :vartype PEN: str
    :cvar REJ: "REJ"
    :vartype REJ: str
    :cvar DIS: "DIS"
    :vartype DIS: str
    :cvar ENA: "ENA"
    :vartype ENA: str
    """

    ACT = "ACT"
    CLO = "CLO"
    ERR = "ERR"
    PEN = "PEN"
    REJ = "REJ"
    DIS = "DIS"
    ENA = "ENA"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, VirtualAccountIssuingStatus._member_map_.values())
        )


@JsonMap({"id_": "id"})
class VirtualAccountIssuing(BaseModel):
    """VirtualAccountIssuing

    :param id_: ID of the virtual account number object. String starting with issuing_., defaults to None
    :type id_: str, optional
    :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit - 45 characters., defaults to None
    :type merchant_reference_id: str, optional
    :param ewallet: ID of the Rapyd Wallet that the virtual account is attached to. String starting with ewallet_., defaults to None
    :type ewallet: str, optional
    :param bank_account: Details about the virtual account., defaults to None
    :type bank_account: dict, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param status: Indicates the status of the virtual account. * ACT (Active) * CLO (Closed)  * ERR (Error)  * PEN (Pending)  * REJ (REjected)  * DIS  * ENA , defaults to None
    :type status: VirtualAccountIssuingStatus, optional
    :param description: Description of the transaction, as defined by the client., defaults to None
    :type description: str, optional
    :param funding_instructions: Issuied virtual account funding instructions., defaults to None
    :type funding_instructions: dict, optional
    :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
    :type currency: str, optional
    :param requested_currency: Currency received by the virtual account after conversion. One of the following: AUD - Australian Dollar EUR - Euro GBP - Pound Sterling HKD - Hong Kong Dollar SGD - Singapore Dollar USD - US Dollar , defaults to None
    :type requested_currency: str, optional
    :param transactions: Issuied virtual account transactions., defaults to None
    :type transactions: List[VirtualAccountTransactionResponse], optional
    """

    def __init__(
        self,
        id_: str = None,
        merchant_reference_id: str = None,
        ewallet: str = None,
        bank_account: dict = None,
        metadata: dict = None,
        status: VirtualAccountIssuingStatus = None,
        description: str = None,
        funding_instructions: dict = None,
        currency: str = None,
        requested_currency: str = None,
        transactions: List[VirtualAccountTransactionResponse] = None,
    ):
        """VirtualAccountIssuing

        :param id_: ID of the virtual account number object. String starting with issuing_., defaults to None
        :type id_: str, optional
        :param merchant_reference_id: Identifier defined by the client for reference purposes. Limit - 45 characters., defaults to None
        :type merchant_reference_id: str, optional
        :param ewallet: ID of the Rapyd Wallet that the virtual account is attached to. String starting with ewallet_., defaults to None
        :type ewallet: str, optional
        :param bank_account: Details about the virtual account., defaults to None
        :type bank_account: dict, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param status: Indicates the status of the virtual account. * ACT (Active) * CLO (Closed)  * ERR (Error)  * PEN (Pending)  * REJ (REjected)  * DIS  * ENA , defaults to None
        :type status: VirtualAccountIssuingStatus, optional
        :param description: Description of the transaction, as defined by the client., defaults to None
        :type description: str, optional
        :param funding_instructions: Issuied virtual account funding instructions., defaults to None
        :type funding_instructions: dict, optional
        :param currency: In transactions without FX, defines the currency of the transaction. Three-letter ISO 4217 code.\<BR\> In FX transactions:\<BR\> * When `fixed_side` is **buy**, it is the currency received in the Rapyd wallet.\<BR\> * When `fixed_side` is **sell**, it is the currency charged to the buyer., defaults to None
        :type currency: str, optional
        :param requested_currency: Currency received by the virtual account after conversion. One of the following: AUD - Australian Dollar EUR - Euro GBP - Pound Sterling HKD - Hong Kong Dollar SGD - Singapore Dollar USD - US Dollar , defaults to None
        :type requested_currency: str, optional
        :param transactions: Issuied virtual account transactions., defaults to None
        :type transactions: List[VirtualAccountTransactionResponse], optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.ewallet = self._define_str("ewallet", ewallet, nullable=True)
        self.bank_account = bank_account
        self.metadata = metadata
        self.status = (
            self._enum_matching(status, VirtualAccountIssuingStatus.list(), "status")
            if status
            else None
        )
        self.description = self._define_str("description", description, nullable=True)
        self.funding_instructions = funding_instructions
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.requested_currency = self._define_str(
            "requested_currency", requested_currency, nullable=True
        )
        self.transactions = self._define_list(
            transactions, VirtualAccountTransactionResponse
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class RefundsGroupPaymentsBody(BaseModel):
    """RefundsGroupPaymentsBody

    :param amount: The amount of the refund, in the currency of the group payment. Decimal. If this parameter is omitted, the entire group payment is refunded., defaults to None
    :type amount: float, optional
    :param group_payment: ID of the group payment that the refund is charged against. String starting with **gp_**.
    :type group_payment: str
    """

    def __init__(self, group_payment: str, amount: float = None):
        """RefundsGroupPaymentsBody

        :param amount: The amount of the refund, in the currency of the group payment. Decimal. If this parameter is omitted, the entire group payment is refunded., defaults to None
        :type amount: float, optional
        :param group_payment: ID of the group payment that the refund is charged against. String starting with **gp_**.
        :type group_payment: str
        """
        self.amount = self._define_number("amount", amount, nullable=True)
        self.group_payment = group_payment



from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .inline_response_200_100_data_application_type import (
    InlineResponse200_100DataApplicationType,
)
from .inline_response_200_100_data_organization_details import (
    InlineResponse200_100DataOrganizationDetails,
)
from .inline_response_200_100_data_renew_result import (
    InlineResponse200_100DataRenewResult,
)


class ApplicationLevel(Enum):
    """An enumeration representing different categories.

    :cvar _1: 1
    :vartype _1: float
    :cvar _2: 2
    :vartype _2: float
    """

    _1 = 1
    _2 = 2

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ApplicationLevel._member_map_.values()))


@JsonMap({})
class InlineResponse200_100Data(BaseModel):
    """InlineResponse200_100Data

    :param token: token, defaults to None
    :type token: str, optional
    :param application_token: application_token, defaults to None
    :type application_token: str, optional
    :param country: country, defaults to None
    :type country: str, optional
    :param rapyd_entity_token: rapyd_entity_token, defaults to None
    :type rapyd_entity_token: str, optional
    :param client_reference_id: client_reference_id, defaults to None
    :type client_reference_id: str, optional
    :param cancel_url: cancel_url, defaults to None
    :type cancel_url: str, optional
    :param complete_url: complete_url, defaults to None
    :type complete_url: str, optional
    :param phone_number: phone_number, defaults to None
    :type phone_number: str, optional
    :param application_level: application_level, defaults to None
    :type application_level: ApplicationLevel, optional
    :param sell_type: sell_type, defaults to None
    :type sell_type: str, optional
    :param business_industry: business_industry, defaults to None
    :type business_industry: str, optional
    :param application_type: application_type, defaults to None
    :type application_type: InlineResponse200_100DataApplicationType, optional
    :param organization_details: organization_details, defaults to None
    :type organization_details: InlineResponse200_100DataOrganizationDetails, optional
    :param renew_result: renew_result, defaults to None
    :type renew_result: InlineResponse200_100DataRenewResult, optional
    """

    def __init__(
        self,
        token: str = None,
        application_token: str = None,
        country: str = None,
        rapyd_entity_token: str = None,
        client_reference_id: str = None,
        cancel_url: str = None,
        complete_url: str = None,
        phone_number: str = None,
        application_level: ApplicationLevel = None,
        sell_type: str = None,
        business_industry: str = None,
        application_type: InlineResponse200_100DataApplicationType = None,
        organization_details: InlineResponse200_100DataOrganizationDetails = None,
        renew_result: InlineResponse200_100DataRenewResult = None,
    ):
        """InlineResponse200_100Data

        :param token: token, defaults to None
        :type token: str, optional
        :param application_token: application_token, defaults to None
        :type application_token: str, optional
        :param country: country, defaults to None
        :type country: str, optional
        :param rapyd_entity_token: rapyd_entity_token, defaults to None
        :type rapyd_entity_token: str, optional
        :param client_reference_id: client_reference_id, defaults to None
        :type client_reference_id: str, optional
        :param cancel_url: cancel_url, defaults to None
        :type cancel_url: str, optional
        :param complete_url: complete_url, defaults to None
        :type complete_url: str, optional
        :param phone_number: phone_number, defaults to None
        :type phone_number: str, optional
        :param application_level: application_level, defaults to None
        :type application_level: ApplicationLevel, optional
        :param sell_type: sell_type, defaults to None
        :type sell_type: str, optional
        :param business_industry: business_industry, defaults to None
        :type business_industry: str, optional
        :param application_type: application_type, defaults to None
        :type application_type: InlineResponse200_100DataApplicationType, optional
        :param organization_details: organization_details, defaults to None
        :type organization_details: InlineResponse200_100DataOrganizationDetails, optional
        :param renew_result: renew_result, defaults to None
        :type renew_result: InlineResponse200_100DataRenewResult, optional
        """
        self.token = self._define_str("token", token, nullable=True)
        self.application_token = self._define_str(
            "application_token", application_token, nullable=True
        )
        self.country = self._define_str("country", country, nullable=True)
        self.rapyd_entity_token = self._define_str(
            "rapyd_entity_token", rapyd_entity_token, nullable=True
        )
        self.client_reference_id = self._define_str(
            "client_reference_id", client_reference_id, nullable=True
        )
        self.cancel_url = self._define_str("cancel_url", cancel_url, nullable=True)
        self.complete_url = self._define_str(
            "complete_url", complete_url, nullable=True
        )
        self.phone_number = self._define_str(
            "phone_number", phone_number, nullable=True
        )
        self.application_level = (
            self._enum_matching(
                application_level, ApplicationLevel.list(), "application_level"
            )
            if application_level
            else None
        )
        self.sell_type = self._define_str("sell_type", sell_type, nullable=True)
        self.business_industry = self._define_str(
            "business_industry", business_industry, nullable=True
        )
        self.application_type = self._define_object(
            application_type, InlineResponse200_100DataApplicationType
        )
        self.organization_details = self._define_object(
            organization_details, InlineResponse200_100DataOrganizationDetails
        )
        self.renew_result = self._define_object(
            renew_result, InlineResponse200_100DataRenewResult
        )



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    InlineResponse200_15,
    InlineResponse200_16,
    InlineResponse200_17,
    InlineResponse200_18,
    InlineResponse200_19,
    SubscriptionItemIdUsageRecordsBody,
    SubscriptionItemsSubscriptionItemIdBody,
    V1SubscriptionItemsBody,
)


class SubscriptionSubscriptionItemService(BaseService):

    @cast_models
    def list_subscription_item(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        ending_before: float = None,
        limit: float = None,
        starting_after: str = None,
        subscription: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_15:
        """Retrieve a list of all subscription items for a subscription.

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param ending_before: The ID of the subscription item created after the last subscription item you want to retrieve., defaults to None
        :type ending_before: float, optional
        :param limit: The maximum number of subscription items to return. Range 1-100. Default is 10., defaults to None
        :type limit: float, optional
        :param starting_after: The ID of the subscription item created before the first subscription item you want to retrieve., defaults to None
        :type starting_after: str, optional
        :param subscription: ID of the subscription., defaults to None
        :type subscription: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List of subscriptions
        :rtype: InlineResponse200_15
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(float).is_optional().validate(ending_before)
        Validator(float).is_optional().validate(limit)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(subscription)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/subscription_items", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("ending_before", ending_before, explode=False)
            .add_query("limit", limit, explode=False)
            .add_query("starting_after", starting_after, explode=False)
            .add_query("subscription", subscription, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_15._unmap(response)

    @cast_models
    def create_subscription_item(
        self,
        request_body: V1SubscriptionItemsBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_16:
        """Create a subscription item and add it to an existing subscription for recurring payment.

        :param request_body: The request body.
        :type request_body: V1SubscriptionItemsBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The created subscription
        :rtype: InlineResponse200_16
        """

        Validator(V1SubscriptionItemsBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/subscription_items", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_16._unmap(response)

    @cast_models
    def retrieve_subscription_item(
        self,
        subscription_item_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_16:
        """Retrieve the details of a subscription item.

        :param subscription_item_id: ID of the subscription item. String starting with **subi_**.
        :type subscription_item_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The created subscription
        :rtype: InlineResponse200_16
        """

        Validator(str).validate(subscription_item_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/subscription_items/{{subscriptionItemId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("subscriptionItemId", subscription_item_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_16._unmap(response)

    @cast_models
    def update_subscription_item(
        self,
        request_body: SubscriptionItemsSubscriptionItemIdBody,
        subscription_item_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_16:
        """Change or modify a subscription item.

        :param request_body: The request body.
        :type request_body: SubscriptionItemsSubscriptionItemIdBody
        :param subscription_item_id: ID of the subscription item. String starting with **subi_**.
        :type subscription_item_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The created subscription
        :rtype: InlineResponse200_16
        """

        Validator(SubscriptionItemsSubscriptionItemIdBody).validate(request_body)
        Validator(str).validate(subscription_item_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/subscription_items/{{subscriptionItemId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("subscriptionItemId", subscription_item_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_16._unmap(response)

    @cast_models
    def delete_subscription_item(
        self,
        subscription_item_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_17:
        """Delete a subscription item from the Rapyd platform.

        :param subscription_item_id: ID of the subscription item. String starting with **subi_**.
        :type subscription_item_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The created subscription
        :rtype: InlineResponse200_17
        """

        Validator(str).validate(subscription_item_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/subscription_items/{{subscriptionItemId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("subscriptionItemId", subscription_item_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_17._unmap(response)

    @cast_models
    def usage_record_summaries(
        self,
        subscription_item_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        limit: float = None,
        ending_before: float = None,
        starting_after: float = None,
        idempotency: str = None,
    ) -> InlineResponse200_18:
        """Retrieve a list of usage records for a subscription item

        :param subscription_item_id: ID of the subscription item. String starting with **subi_**.
        :type subscription_item_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param limit: The maximum number of usage records that are returned. Range is 1-100. Default is 10., defaults to None
        :type limit: float, optional
        :param ending_before: The latest date and time of the returned usage records. Format is in Unix time., defaults to None
        :type ending_before: float, optional
        :param starting_after: The earliest date and time of the returned usage records. Format is in Unix time., defaults to None
        :type starting_after: float, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The subscription after updated
        :rtype: InlineResponse200_18
        """

        Validator(str).validate(subscription_item_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(float).is_optional().validate(limit)
        Validator(float).is_optional().validate(ending_before)
        Validator(float).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/subscription_items/{{subscriptionItemId}}/usage_record_summaries",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("subscriptionItemId", subscription_item_id)
            .add_query("limit", limit)
            .add_query("ending_before", ending_before)
            .add_query("starting_after", starting_after)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_18._unmap(response)

    @cast_models
    def create_subscription_item_usage_record(
        self,
        request_body: SubscriptionItemIdUsageRecordsBody,
        subscription_item_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_19:
        """Create a usage record or update an existing usage record where its unique identifier is composed of timestamp and subscription_item

        :param request_body: The request body.
        :type request_body: SubscriptionItemIdUsageRecordsBody
        :param subscription_item_id: ID of the subscription item. String starting with **subi_**.
        :type subscription_item_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The subscription after updated
        :rtype: InlineResponse200_19
        """

        Validator(SubscriptionItemIdUsageRecordsBody).validate(request_body)
        Validator(str).validate(subscription_item_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/subscription_items/{{subscriptionItemId}}/usage_records",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("subscriptionItemId", subscription_item_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_19._unmap(response)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class V1IdentitiesBody(BaseModel):
    """V1IdentitiesBody

    :param back_side_image: Base-64-encoded image of the back of the identity document., defaults to None
    :type back_side_image: str, optional
    :param back_side_image_mime_type: MIME type of the back side image of the identity document. Default is, defaults to None
    :type back_side_image_mime_type: str, optional
    :param contact: ID of a personal contact for a 'person' wallet., defaults to None
    :type contact: str, optional
    :param country: country
    :type country: str
    :param document_type: Type of the identification document. Two-letter code.
    :type document_type: str
    :param ewallet: ID of the Rapyd Wallet. String starting with ewallet_.
    :type ewallet: str
    :param face_image: Base-64-encoded image of the face.
    :type face_image: str
    :param face_image_mime_type: MIME type of the face image. Default is image/jpeg., defaults to None
    :type face_image_mime_type: str, optional
    :param front_side_image: Base-64-encoded image of the front of the identity document.
    :type front_side_image: str
    :param front_side_image_mime_type: MIME type of the front side image of the identity document. Default is image/jpeg., defaults to None
    :type front_side_image_mime_type: str, optional
    :param reference_id: ID of the identity verification request. Must be unique for each request. Defined by the client. Maximum length: 36 characters. In sandbox, to simulate success or failure, the string must contain one of the following values: success, failure For example: 12345678success
    :type reference_id: str
    :param request_type: Determines the action that is taken on the request., defaults to None
    :type request_type: str, optional
    :param send_callback: Determines whether a webhook is sent with the results of the verification request., defaults to None
    :type send_callback: str, optional
    """

    def __init__(
        self,
        country: str,
        document_type: str,
        ewallet: str,
        face_image: str,
        front_side_image: str,
        reference_id: str,
        back_side_image: str = None,
        back_side_image_mime_type: str = None,
        contact: str = None,
        face_image_mime_type: str = None,
        front_side_image_mime_type: str = None,
        request_type: str = None,
        send_callback: str = None,
    ):
        """V1IdentitiesBody

        :param back_side_image: Base-64-encoded image of the back of the identity document., defaults to None
        :type back_side_image: str, optional
        :param back_side_image_mime_type: MIME type of the back side image of the identity document. Default is, defaults to None
        :type back_side_image_mime_type: str, optional
        :param contact: ID of a personal contact for a 'person' wallet., defaults to None
        :type contact: str, optional
        :param country: country
        :type country: str
        :param document_type: Type of the identification document. Two-letter code.
        :type document_type: str
        :param ewallet: ID of the Rapyd Wallet. String starting with ewallet_.
        :type ewallet: str
        :param face_image: Base-64-encoded image of the face.
        :type face_image: str
        :param face_image_mime_type: MIME type of the face image. Default is image/jpeg., defaults to None
        :type face_image_mime_type: str, optional
        :param front_side_image: Base-64-encoded image of the front of the identity document.
        :type front_side_image: str
        :param front_side_image_mime_type: MIME type of the front side image of the identity document. Default is image/jpeg., defaults to None
        :type front_side_image_mime_type: str, optional
        :param reference_id: ID of the identity verification request. Must be unique for each request. Defined by the client. Maximum length: 36 characters. In sandbox, to simulate success or failure, the string must contain one of the following values: success, failure For example: 12345678success
        :type reference_id: str
        :param request_type: Determines the action that is taken on the request., defaults to None
        :type request_type: str, optional
        :param send_callback: Determines whether a webhook is sent with the results of the verification request., defaults to None
        :type send_callback: str, optional
        """
        self.back_side_image = self._define_str(
            "back_side_image", back_side_image, nullable=True
        )
        self.back_side_image_mime_type = self._define_str(
            "back_side_image_mime_type", back_side_image_mime_type, nullable=True
        )
        self.contact = self._define_str("contact", contact, nullable=True)
        self.country = self._define_str(
            "country",
            country,
            pattern="Name of the country. Two-letter ISO 3166-1 alpha-2 code.",
        )
        self.document_type = document_type
        self.ewallet = ewallet
        self.face_image = face_image
        self.face_image_mime_type = self._define_str(
            "face_image_mime_type", face_image_mime_type, nullable=True
        )
        self.front_side_image = front_side_image
        self.front_side_image_mime_type = self._define_str(
            "front_side_image_mime_type", front_side_image_mime_type, nullable=True
        )
        self.reference_id = reference_id
        self.request_type = self._define_str(
            "request_type", request_type, nullable=True
        )
        self.send_callback = self._define_str(
            "send_callback", send_callback, nullable=True
        )



from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .category import Category
from .entity_type import EntityType
from .gender import Gender


class PayoutsExtendedBeneficiaryBodyIdentificationType(Enum):
    """An enumeration representing different categories.

    :cvar DRIVERSLICENSE: "drivers_license"
    :vartype DRIVERSLICENSE: str
    :cvar IDENTIFICATIONID: "identification_id"
    :vartype IDENTIFICATIONID: str
    :cvar INTERNATIONALPASSPORT: "international_passport"
    :vartype INTERNATIONALPASSPORT: str
    :cvar RESIDENCEPERMIT: "residence_permit"
    :vartype RESIDENCEPERMIT: str
    :cvar SOCIALSECURITY: "social_security"
    :vartype SOCIALSECURITY: str
    :cvar WORKPERMIT: "work_permit"
    :vartype WORKPERMIT: str
    """

    DRIVERSLICENSE = "drivers_license"
    IDENTIFICATIONID = "identification_id"
    INTERNATIONALPASSPORT = "international_passport"
    RESIDENCEPERMIT = "residence_permit"
    SOCIALSECURITY = "social_security"
    WORKPERMIT = "work_permit"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                PayoutsExtendedBeneficiaryBodyIdentificationType._member_map_.values(),
            )
        )


@JsonMap({})
class PayoutsExtendedBeneficiaryBody(BaseModel):
    """PayoutsExtendedBeneficiaryBody

    :param address: Beneficiary's street address including the build number.
    :type address: str
    :param city: City of the beneficiary.
    :type city: str
    :param category: category
    :type category: Category
    :param company_name: Name of the beneficiary company. Required when `entity_type` is **company**., defaults to None
    :type company_name: str, optional
    :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code.
    :type country: str
    :param country_of_incorporation: The country where the company was registered. Two-letter ISO 3166-1 ALPHA-2 code. Required when `entity_type` is **company**., defaults to None
    :type country_of_incorporation: str, optional
    :param currency: currency
    :type currency: str
    :param date_of_birth: Date of birth of the individual. Format: DD/MM/YYYY. Required when `entity_type` is **individual**., defaults to None
    :type date_of_birth: str, optional
    :param date_of_incorporation: The date when the company was registered. Format: DD/MM/YYYY. Required when `entity_type` is **company**., defaults to None
    :type date_of_incorporation: str, optional
    :param default_payout_method_type: The type of payout method for the beneficiary. The two-letter prefix must match the beneficiary country code., defaults to None
    :type default_payout_method_type: str, optional
    :param entity_type: entity_type
    :type entity_type: EntityType
    :param first_name: First name of the beneficiary. Required when `entity_type` is **individual**., defaults to None
    :type first_name: str, optional
    :param gender: Gender of the individual. Required when `entity_type` is **individual**., defaults to None
    :type gender: Gender, optional
    :param identification_type: Type of identification document for the beneficiary. When `entity_type` is **company**, this field must be**company_registered_number**. When `entity_type` is **individual**:
    :type identification_type: PayoutsExtendedBeneficiaryBodyIdentificationType
    :param identification_value: Identification number on the document mentioned in `identification_type`.
    :type identification_value: str
    :param last_name: Family name of the beneficiary. Required when `entity_type` is **individual**. Required when `entity_type` is **individual**., defaults to None
    :type last_name: str, optional
    :param merchant_reference_id: ID defined by the client., defaults to None
    :type merchant_reference_id: str, optional
    :param nationality: The citizenship of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code for the country. To determine the code for a country, see 'List Countries'. Required when `entity_type` is **individual**., defaults to None
    :type nationality: str, optional
    """

    def __init__(
        self,
        address: str,
        city: str,
        category: Category,
        country: str,
        currency: str,
        entity_type: EntityType,
        identification_type: PayoutsExtendedBeneficiaryBodyIdentificationType,
        identification_value: str,
        company_name: str = None,
        country_of_incorporation: str = None,
        date_of_birth: str = None,
        date_of_incorporation: str = None,
        default_payout_method_type: str = None,
        first_name: str = None,
        gender: Gender = None,
        last_name: str = None,
        merchant_reference_id: str = None,
        nationality: str = None,
    ):
        """PayoutsExtendedBeneficiaryBody

        :param address: Beneficiary's street address including the build number.
        :type address: str
        :param city: City of the beneficiary.
        :type city: str
        :param category: category
        :type category: Category
        :param company_name: Name of the beneficiary company. Required when `entity_type` is **company**., defaults to None
        :type company_name: str, optional
        :param country: Country of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code. The two-letter prefix of the payout method type must match the beneficiary country code.
        :type country: str
        :param country_of_incorporation: The country where the company was registered. Two-letter ISO 3166-1 ALPHA-2 code. Required when `entity_type` is **company**., defaults to None
        :type country_of_incorporation: str, optional
        :param currency: currency
        :type currency: str
        :param date_of_birth: Date of birth of the individual. Format: DD/MM/YYYY. Required when `entity_type` is **individual**., defaults to None
        :type date_of_birth: str, optional
        :param date_of_incorporation: The date when the company was registered. Format: DD/MM/YYYY. Required when `entity_type` is **company**., defaults to None
        :type date_of_incorporation: str, optional
        :param default_payout_method_type: The type of payout method for the beneficiary. The two-letter prefix must match the beneficiary country code., defaults to None
        :type default_payout_method_type: str, optional
        :param entity_type: entity_type
        :type entity_type: EntityType
        :param first_name: First name of the beneficiary. Required when `entity_type` is **individual**., defaults to None
        :type first_name: str, optional
        :param gender: Gender of the individual. Required when `entity_type` is **individual**., defaults to None
        :type gender: Gender, optional
        :param identification_type: Type of identification document for the beneficiary. When `entity_type` is **company**, this field must be**company_registered_number**. When `entity_type` is **individual**:
        :type identification_type: PayoutsExtendedBeneficiaryBodyIdentificationType
        :param identification_value: Identification number on the document mentioned in `identification_type`.
        :type identification_value: str
        :param last_name: Family name of the beneficiary. Required when `entity_type` is **individual**. Required when `entity_type` is **individual**., defaults to None
        :type last_name: str, optional
        :param merchant_reference_id: ID defined by the client., defaults to None
        :type merchant_reference_id: str, optional
        :param nationality: The citizenship of the beneficiary. Two-letter ISO 3166-1 ALPHA-2 code for the country. To determine the code for a country, see 'List Countries'. Required when `entity_type` is **individual**., defaults to None
        :type nationality: str, optional
        """
        self.address = address
        self.city = city
        self.category = self._enum_matching(category, Category.list(), "category")
        self.company_name = self._define_str(
            "company_name", company_name, nullable=True
        )
        self.country = country
        self.country_of_incorporation = self._define_str(
            "country_of_incorporation", country_of_incorporation, nullable=True
        )
        self.currency = self._define_str(
            "currency",
            currency,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.date_of_birth = self._define_str(
            "date_of_birth", date_of_birth, nullable=True
        )
        self.date_of_incorporation = self._define_str(
            "date_of_incorporation", date_of_incorporation, nullable=True
        )
        self.default_payout_method_type = self._define_str(
            "default_payout_method_type", default_payout_method_type, nullable=True
        )
        self.entity_type = self._enum_matching(
            entity_type, EntityType.list(), "entity_type"
        )
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.gender = (
            self._enum_matching(gender, Gender.list(), "gender") if gender else None
        )
        self.identification_type = self._enum_matching(
            identification_type,
            PayoutsExtendedBeneficiaryBodyIdentificationType.list(),
            "identification_type",
        )
        self.identification_value = identification_value
        self.last_name = self._define_str("last_name", last_name, nullable=True)
        self.merchant_reference_id = self._define_str(
            "merchant_reference_id", merchant_reference_id, nullable=True
        )
        self.nationality = self._define_str("nationality", nationality, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class SubscriptionItemIdUsageRecordsBody(BaseModel):
    """SubscriptionItemIdUsageRecordsBody

    :param action: Determines how the quantity is defined in the usage record., defaults to None
    :type action: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param quantity: The usage quantity for the specified date and time (timestamp).
    :type quantity: float
    :param timestamp: Timestamp for the usage record in Unix time.
    :type timestamp: float
    """

    def __init__(
        self,
        quantity: float,
        timestamp: float,
        action: str = None,
        metadata: dict = None,
    ):
        """SubscriptionItemIdUsageRecordsBody

        :param action: Determines how the quantity is defined in the usage record., defaults to None
        :type action: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param quantity: The usage quantity for the specified date and time (timestamp).
        :type quantity: float
        :param timestamp: Timestamp for the usage record in Unix time.
        :type timestamp: float
        """
        self.action = self._define_str("action", action, nullable=True)
        self.metadata = metadata
        self.quantity = quantity
        self.timestamp = timestamp



from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


class InlineResponse200_88DataStatus(Enum):
    """An enumeration representing different categories.

    :cvar NEW: "NEW"
    :vartype NEW: str
    :cvar DON: "DON"
    :vartype DON: str
    :cvar EXP: "EXP"
    :vartype EXP: str
    """

    NEW = "NEW"
    DON = "DON"
    EXP = "EXP"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, InlineResponse200_88DataStatus._member_map_.values())
        )


@JsonMap({})
class InlineResponse200_88Data(BaseModel):
    """InlineResponse200_88Data

    :param status: status, defaults to None
    :type status: InlineResponse200_88DataStatus, optional
    """

    def __init__(self, status: InlineResponse200_88DataStatus = None):
        """InlineResponse200_88Data

        :param status: status, defaults to None
        :type status: InlineResponse200_88DataStatus, optional
        """
        self.status = (
            self._enum_matching(status, InlineResponse200_88DataStatus.list(), "status")
            if status
            else None
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class V1hosteddisbursebeneficiaryBeneficiaryOptionalFields(BaseModel):
    """V1hosteddisbursebeneficiaryBeneficiaryOptionalFields

    :param company_name: The name of the beneficiary's company. Relevant to beneficiaries that are not individuals. If the request does not contain this information, the hosted page prompts the customer for it. \<BR\> If `beneficiary_entity_type` is **individual**, this field must be omitted., defaults to None
    :type company_name: str, optional
    :param first_name: The first name of the beneficiary. Relevant to individual beneficiaries. If the request does not contain this information, the redirect page prompts the customer for it., defaults to None
    :type first_name: str, optional
    :param identification_type: Type of identification document. When `entity_type` is **company**, this field must be **company_registered_number**. When `entity_type` is **individual**, one of the following values: \<BR\> * **drivers_license**\<BR\> * **identification_id** \<BR\> * **international_passport** \<BR\> * **residence_permit**\<BR\> * **social_security**\<BR\> * **work_permit** type: string
    :type identification_type: any
    :param identification_value: The identification of the document mentioned in identification_type.
    :type identification_value: str
    :param last_name: The last name of the beneficiary. Relevant to individual beneficiaries. If the request does not contain this information, the redirect page prompts the customer for it., defaults to None
    :type last_name: str, optional
    """

    def __init__(
        self,
        identification_type: any,
        identification_value: str,
        company_name: str = None,
        first_name: str = None,
        last_name: str = None,
    ):
        """V1hosteddisbursebeneficiaryBeneficiaryOptionalFields

        :param company_name: The name of the beneficiary's company. Relevant to beneficiaries that are not individuals. If the request does not contain this information, the hosted page prompts the customer for it. \<BR\> If `beneficiary_entity_type` is **individual**, this field must be omitted., defaults to None
        :type company_name: str, optional
        :param first_name: The first name of the beneficiary. Relevant to individual beneficiaries. If the request does not contain this information, the redirect page prompts the customer for it., defaults to None
        :type first_name: str, optional
        :param identification_type: Type of identification document. When `entity_type` is **company**, this field must be **company_registered_number**. When `entity_type` is **individual**, one of the following values: \<BR\> * **drivers_license**\<BR\> * **identification_id** \<BR\> * **international_passport** \<BR\> * **residence_permit**\<BR\> * **social_security**\<BR\> * **work_permit** type: string
        :type identification_type: any
        :param identification_value: The identification of the document mentioned in identification_type.
        :type identification_value: str
        :param last_name: The last name of the beneficiary. Relevant to individual beneficiaries. If the request does not contain this information, the redirect page prompts the customer for it., defaults to None
        :type last_name: str, optional
        """
        self.company_name = self._define_str(
            "company_name", company_name, nullable=True
        )
        self.first_name = self._define_str("first_name", first_name, nullable=True)
        self.identification_type = identification_type
        self.identification_value = identification_value
        self.last_name = self._define_str("last_name", last_name, nullable=True)



from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .invoice_item_period import InvoiceItemPeriod
from .plan import Plan


@JsonMap({"id_": "id", "type_": "type"})
class InvoiceItem(BaseModel):
    """Invoice item

    :param id_: line item id, defaults to None
    :type id_: str, optional
    :param amount: amount, defaults to None
    :type amount: float, optional
    :param currency: currency, defaults to None
    :type currency: str, optional
    :param description: item description, defaults to None
    :type description: str, optional
    :param discountable: discountable, defaults to None
    :type discountable: bool, optional
    :param invoice_item: invoice item id, defaults to None
    :type invoice_item: str, optional
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param period: period, defaults to None
    :type period: InvoiceItemPeriod, optional
    :param proration: proration, defaults to None
    :type proration: bool, optional
    :param quantity: quantity, defaults to None
    :type quantity: float, optional
    :param plan: plan, defaults to None
    :type plan: Plan, optional
    :param subscription: ID of the subscription that generates charges to this customer. String starting with sub_., defaults to None
    :type subscription: str, optional
    :param subscription_item: ID of the subscription item that generates charges to this customer., defaults to None
    :type subscription_item: str, optional
    :param type_: type_, defaults to None
    :type type_: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        amount: float = None,
        currency: str = None,
        description: str = None,
        discountable: bool = None,
        invoice_item: str = None,
        metadata: dict = None,
        period: InvoiceItemPeriod = None,
        proration: bool = None,
        quantity: float = None,
        plan: Plan = None,
        subscription: str = None,
        subscription_item: str = None,
        type_: str = None,
    ):
        """Invoice item

        :param id_: line item id, defaults to None
        :type id_: str, optional
        :param amount: amount, defaults to None
        :type amount: float, optional
        :param currency: currency, defaults to None
        :type currency: str, optional
        :param description: item description, defaults to None
        :type description: str, optional
        :param discountable: discountable, defaults to None
        :type discountable: bool, optional
        :param invoice_item: invoice item id, defaults to None
        :type invoice_item: str, optional
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param period: period, defaults to None
        :type period: InvoiceItemPeriod, optional
        :param proration: proration, defaults to None
        :type proration: bool, optional
        :param quantity: quantity, defaults to None
        :type quantity: float, optional
        :param plan: plan, defaults to None
        :type plan: Plan, optional
        :param subscription: ID of the subscription that generates charges to this customer. String starting with sub_., defaults to None
        :type subscription: str, optional
        :param subscription_item: ID of the subscription item that generates charges to this customer., defaults to None
        :type subscription_item: str, optional
        :param type_: type_, defaults to None
        :type type_: str, optional
        """
        self.id_ = self._define_str("id_", id_, nullable=True)
        self.amount = self._define_number("amount", amount, nullable=True)
        self.currency = self._define_str(
            "currency",
            currency,
            nullable=True,
            pattern="/^AED|AFN|ALL|AMD|ANG|AOA|ARS|AUD|AWG|AZN|BAM|BBD|BDT|BGN|BHD|BIF|BMD|BND|BOB|BRL|BSD|BTN|BWP|BYR|BZD|CAD|CDF|CHF|CLP|CNY|COP|CRC|CUC|CUP|CVE|CZK|DJF|DKK|DOP|DZD|EGP|ERN|ETB|EUR|FJD|FKP|GBP|GEL|GGP|GHS|GIP|GMD|GNF|GTQ|GYD|HKD|HNL|HRK|HTG|HUF|IDR|ILS|IMP|INR|IQD|IRR|ISK|JEP|JMD|JOD|JPY|KES|KGS|KHR|KMF|KPW|KRW|KWD|KYD|KZT|LAK|LBP|LKR|LRD|LSL|LYD|MAD|MDL|MGA|MKD|MMK|MNT|MOP|MRO|MUR|MVR|MWK|MXN|MYR|MZN|NAD|NGN|NIO|NOK|NPR|NZD|OMR|PAB|PEN|PGK|PHP|PKR|PLN|PYG|QAR|RON|RSD|RUB|RWF|SAR|SBD|SCR|SDG|SEK|SGD|SHP|SLL|SOS|SPL|SRD|STD|SVC|SYP|SZL|THB|TJS|TMT|TND|TOP|TRY|TTD|TVD|TWD|TZS|UAH|UGX|USD|UYU|UZS|VEF|VND|VUV|WST|XAF|XCD|XDR|XOF|XPF|YER|ZAR|ZMW|ZWD$/",
        )
        self.description = self._define_str("description", description, nullable=True)
        self.discountable = discountable
        self.invoice_item = self._define_str(
            "invoice_item", invoice_item, nullable=True
        )
        self.metadata = metadata
        self.period = self._define_object(period, InvoiceItemPeriod)
        self.proration = proration
        self.quantity = self._define_number("quantity", quantity, nullable=True)
        self.plan = self._define_object(plan, Plan)
        self.subscription = self._define_str(
            "subscription", subscription, nullable=True
        )
        self.subscription_item = self._define_str(
            "subscription_item", subscription_item, nullable=True
        )
        self.type_ = self._define_str("type_", type_, nullable=True)



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    ApplicationsHostedBody,
    HostedIdvBody,
    InlineResponse200_111,
    InlineResponse200_112,
    InlineResponse200_113,
    InlineResponse200_114,
    InlineResponse200_115,
    InlineResponse200_116,
    InlineResponse200_117,
    V1IdentitiesBody,
)


class VerifyService(BaseService):

    @cast_models
    def verify_identity(
        self,
        request_body: V1IdentitiesBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_111:
        """Verification of the identity of a personal contact associated with a wallet.

        :param request_body: The request body.
        :type request_body: V1IdentitiesBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Retrieve the Rapyd ID and merchant reference ID.
        :rtype: InlineResponse200_111
        """

        Validator(V1IdentitiesBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/identities", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_111._unmap(response)

    @cast_models
    def get_kyc_id_verification_supported_doc_types(
        self,
        country: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_112:
        """Use this method to determine the types of documents to use for identification purposes, and also whether you need the reverse side of the document. You can filter your search results by country with the country query parameter.

        :param country: Two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase.
        :type country: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List Contacts for a Rapyd Wallet
        :rtype: InlineResponse200_112
        """

        Validator(str).pattern(
            "Name of the country. Two-letter ISO 3166-1 alpha-2 code."
        ).validate(country)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/identities/types", self.get_default_headers()
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("country", country, explode=False)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_112._unmap(response)

    @cast_models
    def get_application_types_by_country(
        self,
        country: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_113:
        """Retrieve a List types of identity verification applications by country. You can filter the list with query parameters.

        :param country: Two-letter ISO 3166-1 ALPHA-2 code for the country. Uppercase.
        :type country: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List types of identity verification applications by country.
        :rtype: InlineResponse200_113
        """

        Validator(str).pattern(
            "Name of the country. Two-letter ISO 3166-1 alpha-2 code."
        ).validate(country)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/verify/applications/types/country/{{country}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("country", country)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_113._unmap(response)

    @cast_models
    def get_application_status(
        self,
        application: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_114:
        """Retrieve the status of an application for verification of identity. You can filter the list with query parameters.

        :param application: ID of the application. String starting with app_.
        :type application: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: List types of identity verification applications by country.
        :rtype: InlineResponse200_114
        """

        Validator(str).validate(application)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/verify/applications/status/{{application}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("application", application)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_114._unmap(response)

    @cast_models
    def create_hosted_application_token(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: ApplicationsHostedBody = None,
        idempotency: str = None,
    ) -> InlineResponse200_115:
        """Create an application for Rapyd Verify where the client (applicant) provides information directly to Rapyd. After the applicant submits the application on the hosted page, Rapyd verifies the information. When the status of the application changes, Rapyd sends you a webhook.

        :param request_body: The request body., defaults to None
        :type request_body: ApplicationsHostedBody, optional
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Hosted application details.
        :rtype: InlineResponse200_115
        """

        Validator(ApplicationsHostedBody).is_optional().validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/verify/applications/hosted",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_115._unmap(response)

    @cast_models
    def get_hosted_application_by_token(
        self,
        verify_app_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_116:
        """Retrieve Rapyd Verify Application

        :param verify_app_id: ID of the Rapyd Verify application. String starting with happ_.
        :type verify_app_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Hosted application details.
        :rtype: InlineResponse200_116
        """

        Validator(str).validate(verify_app_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/verify/applications/hosted/{{verifyAppId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("verifyAppId", verify_app_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_116._unmap(response)

    @cast_models
    def identity_verification_page(
        self,
        request_body: HostedIdvBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_117:
        """Create a hosted page for verifying the identity of a personal contact for a Rapyd Wallet.<BR>The customer completes the process by accessing the URL returned in the response.<BR>After the customer submits the identity verification request on the hosted page, Rapyd verifies the information and sends you a webhook with a summary of the identity check. See:<BR>* Identity Verification Webhook - Sandbox Approved<BR>* Identity Verification Webhook - Production Approved<BR>* Identity Verification Webhook - Production Not Approved<BR>**Prerequisites**<BR>* Rapyd Wallet

        :param request_body: The request body.
        :type request_body: HostedIdvBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Returns details on the Identity Verification Page.
        :rtype: InlineResponse200_117
        """

        Validator(HostedIdvBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/hosted/idv", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_117._unmap(response)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class PaymentSteps(BaseModel):
    """PaymentSteps

    :param step: step, defaults to None
    :type step: str, optional
    """

    def __init__(self, step: str = None):
        """PaymentSteps

        :param step: step, defaults to None
        :type step: str, optional
        """
        self.step = self._define_str("step", step, nullable=True)



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({})
class InvoiceIdPayBody(BaseModel):
    """InvoiceIdPayBody

    :param payment_method: ID of the payment method for paying the invoice. If not included in the body, then the customer's default_payment_method is used., defaults to None
    :type payment_method: str, optional
    """

    def __init__(self, payment_method: str = None):
        """InvoiceIdPayBody

        :param payment_method: ID of the payment method for paying the invoice. If not included in the body, then the customer's default_payment_method is used., defaults to None
        :type payment_method: str, optional
        """
        self.payment_method = self._define_str(
            "payment_method", payment_method, nullable=True
        )



from .utils.json_map import JsonMap
from .utils.base_model import BaseModel


@JsonMap({"id_": "id"})
class TransferResponseBody(BaseModel):
    """TransferResponseBody

    :param id_: ID of the transfer transaction, from the `id` field in the `data` object of the response. 32-digit hexadecimal.
    :type id_: str
    :param metadata: A JSON object defined by the client., defaults to None
    :type metadata: dict, optional
    :param status: Determines how to handle the transfer. One of the following values - accept, decline, cancel
    :type status: str
    """

    def __init__(self, id_: str, status: str, metadata: dict = None):
        """TransferResponseBody

        :param id_: ID of the transfer transaction, from the `id` field in the `data` object of the response. 32-digit hexadecimal.
        :type id_: str
        :param metadata: A JSON object defined by the client., defaults to None
        :type metadata: dict, optional
        :param status: Determines how to handle the transfer. One of the following values - accept, decline, cancel
        :type status: str
        """
        self.id_ = id_
        self.metadata = metadata
        self.status = status



from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models import (
    CustomerRequest,
    InlineResponse200_23,
    InlineResponse200_36,
    InlineResponse200_37,
    InlineResponse200_38,
    InlineResponse200_39,
    V1CustomersBody,
)


class CustomerService(BaseService):

    @cast_models
    def list_customer(
        self,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        starting_after: str = None,
        ending_before: str = None,
        limit: str = None,
        idempotency: str = None,
    ) -> InlineResponse200_36:
        """Retrieve a list of all customers

        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param starting_after: The ID of the customer created before the first customer you want to retrieve, defaults to None
        :type starting_after: str, optional
        :param ending_before: The ID of the customer created after the last customer you want to retrieve, defaults to None
        :type ending_before: str, optional
        :param limit: The maximum number of customers to return. Range is 1-100. Default is 10, defaults to None
        :type limit: str, optional
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: get customers details.
        :rtype: InlineResponse200_36
        """

        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(starting_after)
        Validator(str).is_optional().validate(ending_before)
        Validator(str).is_optional().pattern("([1-9]|[1-9]\d|100)").validate(limit)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/customers", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_query("starting_after", starting_after)
            .add_query("ending_before", ending_before)
            .add_query("limit", limit)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_36._unmap(response)

    @cast_models
    def create_customer(
        self,
        request_body: V1CustomersBody,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_37:
        """Adds a customer to the client.

        :param request_body: The request body.
        :type request_body: V1CustomersBody
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Customer was created successfully.
        :rtype: InlineResponse200_37
        """

        Validator(V1CustomersBody).validate(request_body)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(f"{self.base_url}/v1/customers", self.get_default_headers())
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_37._unmap(response)

    @cast_models
    def retrieve_customer(
        self,
        customer_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_37:
        """Retrieve a customer.

        :param customer_id: customer Id
        :type customer_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Customer details by customer ID.
        :rtype: InlineResponse200_37
        """

        Validator(str).validate(customer_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/customers/{{customerId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("customerId", customer_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_37._unmap(response)

    @cast_models
    def update_customer(
        self,
        customer_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        request_body: CustomerRequest = None,
        idempotency: str = None,
    ) -> InlineResponse200_37:
        """Update a customer with customer Id in Path

        :param request_body: The request body., defaults to None
        :type request_body: CustomerRequest, optional
        :param customer_id: customer Id
        :type customer_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: customer was updated successfully
        :rtype: InlineResponse200_37
        """

        Validator(CustomerRequest).is_optional().validate(request_body)
        Validator(str).validate(customer_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/customers/{{customerId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("customerId", customer_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_37._unmap(response)

    @cast_models
    def delete_customer(
        self,
        customer_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_23:
        """This method triggers the Customer Deleted webhook. This webhook contains the same information as the response

        :param customer_id: customer Id
        :type customer_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: customer was deleted
        :rtype: InlineResponse200_23
        """

        Validator(str).validate(customer_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/customers/{{customerId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("customerId", customer_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_23._unmap(response)

    @cast_models
    def get_customer_discount_by_id(
        self,
        discount_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_38:
        """Retrieve a discount for a customer.

        :param discount_id: discount Id
        :type discount_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Get discount details of a customer by Discount Id.
        :rtype: InlineResponse200_38
        """

        Validator(str).validate(discount_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/customers/discount/{{discountId}}",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("discountId", discount_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_38._unmap(response)

    @cast_models
    def delete_customer_discount(
        self,
        customer_id: str,
        access_key: str,
        content_type: str,
        salt: str,
        signature: str,
        timestamp: str,
        idempotency: str = None,
    ) -> InlineResponse200_39:
        """Delete the discount that has been assigned to a customer through a coupon. This action does not affect the coupon that the discount was derived from. This method triggers the **Customer Discount Deleted** webhook.

        :param customer_id: customer Id
        :type customer_id: str
        :param access_key: Unique access key provided by Rapyd for each authorized user.
        :type access_key: str
        :param content_type: Indicates that the data appears in JSON format. Set to **application/json**.
        :type content_type: str
        :param salt: Random string. Recommended length: 8-16 characters.
        :type salt: str
        :param signature: Signature calculated for each request individually. See [Request Signatures](https://docs.rapyd.net/en/request-signatures.html).
        :type signature: str
        :param timestamp: Timestamp for the request, in Unix time (seconds).
        :type timestamp: str
        :param idempotency: A unique key that prevents the platform from creating the same object twice., defaults to None
        :type idempotency: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: discount was deleted
        :rtype: InlineResponse200_39
        """

        Validator(str).validate(customer_id)
        Validator(str).validate(access_key)
        Validator(str).validate(content_type)
        Validator(str).validate(salt)
        Validator(str).validate(signature)
        Validator(str).validate(timestamp)
        Validator(str).is_optional().validate(idempotency)

        serialized_request = (
            Serializer(
                f"{self.base_url}/v1/customers/{{customerId}}/discount",
                self.get_default_headers(),
            )
            .add_header("access_key", access_key)
            .add_header("Content-Type", content_type)
            .add_header("idempotency", idempotency)
            .add_header("salt", salt)
            .add_header("signature", signature)
            .add_header("timestamp", timestamp)
            .add_path("customerId", customer_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)
        return InlineResponse200_39._unmap(response)
